#!/usr/bin/env python
import inspect
import sys
from isclassmethod import isclassmethod
from isstaticmethod import isstaticmethod
from public import public


def isproperty(obj):
    return isinstance(obj, property)


def getmembers(obj, predicate):
    # python2.6 inspect.getmembers raise error
    # abc.ABCMeta
    # AttributeError: __abstractmethods__
    try:
        return inspect.getmembers(obj, predicate)
    except Exception:
        return []


def check_obj(obj):
    if not obj:
        return None
    if inspect.ismodule(obj):
        return None
    if inspect.isclass(obj):
        return None
    return True


def get_property_class(obj):
    for _, m in sys.modules.items():
        for _, cls in getmembers(m, inspect.isclass):
            for _, _property in getmembers(cls, isproperty):
                if _property == obj:
                    return cls


@public
def getclass(obj):
    """return method cls"""
    if not check_obj(obj):
        return None
    if inspect.ismethod(obj):
        try:
            return obj.im_class  # python2
        except AttributeError:
            return obj.__self__  # python3

    if isinstance(obj, property):  # property
        return get_property_class(obj)
    if isclassmethod(obj) or isstaticmethod(obj):
        module = inspect.getmodule(obj)
        if not module:
            return
        for name in dir(module):
            attr = getattr(module, name)
            if attr is None:
                continue
            if not inspect.isclass(attr):
                continue
            cls = attr
            if hasattr(cls, obj.__name__):
                if obj == getattr(cls, obj.__name__):
                    return cls
    if hasattr(obj, "__class__") and not inspect.isroutine(obj):
        return obj.__class__
